#######
# Framed! v1.1
# By: Michael A. Hawker
# Copyright 2008 Mikeware
#######
# Minimalist entry for Ludum Dare 11 - 48 Hour Competition
# The following code is for contest entry purposes, it is not to be modified
# and/or distributed
#######

import pygame
from pygame.locals import *

from random import randint, choice, shuffle
from math import floor

class Frame:
    def __init__(self, x, y, size):
        self.x = x
        self.y = y
        self.size = size
        self.rect = Rect(x, y, 5*size, 7*size)
        R = randint(192, 255)
        self.color = (R, R, R)
        self.width = 5*size
        self.value = self.width / 2
        self.closing = False
        self.selected = False
        self.other = False
    #end __init__
#end Frame

WIDTH = 640
HEIGHT = 480
FX = int(floor(WIDTH / 18) + 1)
FY = int(floor(HEIGHT / 24) + 1)
PADX = (WIDTH - (FX - 1)*18) / 2
PADY = (HEIGHT - (FY - 1)*24) / 2
SIZES = [1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 5, 6]

# Start Pygame
pygame.mixer.pre_init(22050, -16, 3, 3072)
pygame.init()

# Load Sounds
sndcorrect = pygame.mixer.Sound("sounds/correct.ogg")
sndincorrect = pygame.mixer.Sound("sounds/incorrect.ogg")
sndbad = pygame.mixer.Sound("sounds/bad.ogg")
sndstart = pygame.mixer.Sound("sounds/start.ogg")
sndwin = pygame.mixer.Sound("sounds/win.ogg")
sndlose = pygame.mixer.Sound("sounds/lose.ogg")

# load and start music
try:
    pygame.mixer.music.load("music/framed.ogg")
    pygame.mixer.music.play(-1) # loop forever
    pygame.mixer.music.set_volume(0.4)
except:
    pass
#end try

# display
pygame.display.set_caption("Framed! v1.1")
pygame.display.set_icon(pygame.image.load("graphics/icon.png"))
screen = pygame.display.set_mode((WIDTH, HEIGHT))

# load title image
imgtitle = pygame.image.load("graphics/title.png").convert()
imgwin = pygame.image.load("graphics/win.png").convert_alpha()
imglose = pygame.image.load("graphics/lose.png").convert_alpha()
# load btn images
imgbtnplay = pygame.image.load("graphics/btnplay.png").convert()
imgbtnquit = pygame.image.load("graphics/btnquit.png").convert()
imgbtneasy = [pygame.image.load("graphics/btneasy_green.png").convert(), pygame.image.load("graphics/btneasy_brown.png").convert()]
imgbtnhard = [pygame.image.load("graphics/btnhard_green.png").convert(), pygame.image.load("graphics/btnhard_brown.png").convert()]

# menu button init
sfrplay = Rect((12, 171), imgbtnplay.get_size())
sfrquit = Rect((12, 303), imgbtnquit.get_size())
sfreasy = Rect((106, 171), imgbtneasy[0].get_size())
sfrhard = Rect((106, 236), imgbtnhard[0].get_size())
gamemode = "easy"

# draws a frame of size*6 x size*8, returns True if Frame should be deleted
def draw_frame(surf, fr, all=True, outline=False):
    if outline:
        pygame.draw.rect(surf, (0, 0, 0), fr.rect, 1)
    else:
        if all:
            if fr.value:
                if fr.closing:
                    fr.value += 1
                    if fr.value >= fr.width/2: return True
                else:
                    fr.value -= 1
                #end if
                pygame.draw.rect(surf, (0, 0, 0), Rect(fr.value+fr.x, fr.y, 5*fr.size-2*fr.value, 7*fr.size))
            else:
                pygame.draw.rect(surf, (0, 0, 0), fr.rect)
            #end if
            if fr.selected:
                pygame.draw.rect(surf, (0, 128, 0), Rect(fr.x+1, fr.y+1, 5*fr.size-2, 7*fr.size-2))
            else:
                pygame.draw.rect(surf, fr.color, Rect(fr.x+1+fr.value, fr.y+1, 5*fr.size-2-2*fr.value, 7*fr.size-2))
            #end if
            pygame.draw.rect(surf, (0, 0, 0), Rect(fr.x+1+fr.size+fr.value*0.65, fr.y+1+fr.size, 3*fr.size-2-1.3*fr.value, 5*fr.size-2))
        else:
            #need to just do frame border (selection for menu)
            c = fr.color
            if fr.selected: c = (0, 128, 0)
            pygame.draw.rect(surf, c, Rect(fr.x+1, fr.y+1, 5*fr.size-2, fr.size)) # top rect
            pygame.draw.rect(surf, c, Rect(fr.x+1, fr.y+6*fr.size-1, 5*fr.size-2, fr.size)) # bottom rect
            pygame.draw.rect(surf, c, Rect(fr.x+1, fr.y+fr.size+1, fr.size, 5*fr.size-2)) # left rect
            pygame.draw.rect(surf, c, Rect(fr.x-1+4*fr.size, fr.y+fr.size+1, fr.size, 5*fr.size-2)) # right rect
        #end if
    #end if

    return False
#end draw_frame

# fills in a set of buckets
def fill_buckets(buck, x, y, size):
    for w in range(size):
        for h in range(size):
            buck[x+w][y+h] = True
        #next h
    #next w
#end fill_buckets

# checks if all buckets free
def free_buckets(buck, x, y, size):
    if x+size>=FX or y+size>=FY: return min(FX - x, FY - y)
    for w in range(size):
        for h in range(size):
            if buck[x+w][y+h]: return 0
        #next h
    #next w
    return -1
#end free_buckets

# main menu
notexit = True
while notexit:
    menu = True
    
    # init menu vars
    selplay = False
    selquit = False
    seleasy = False
    selhard = False

    while menu and notexit:
        for evt in pygame.event.get():
            if evt.type == QUIT or (evt.type == KEYDOWN and evt.key == K_ESCAPE):
                notexit = False
            elif evt.type == MOUSEMOTION:
                selplay = sfrplay.collidepoint(evt.pos)
                selquit = sfrquit.collidepoint(evt.pos)
                seleasy = sfreasy.collidepoint(evt.pos)
                selhard = sfrhard.collidepoint(evt.pos)
            elif evt.type == MOUSEBUTTONDOWN:
                if sfrplay.collidepoint(evt.pos):
                    # start game
                    sndstart.play()
                    menu = False
                elif sfrquit.collidepoint(evt.pos):
                    notexit = False
                elif sfreasy.collidepoint(evt.pos):
                    gamemode = "easy"
                elif sfrhard.collidepoint(evt.pos):
                    gamemode = "hard"
                #end if
            #end if
        #next

        screen.blit(imgtitle, (0,0))
        
        if gamemode == "easy":
            screen.blit(imgbtneasy[1], sfreasy.topleft)
        else:
            screen.blit(imgbtnhard[1], sfrhard.topleft)
        #end if
        
        if selplay:
            screen.blit(imgbtnplay, sfrplay.topleft)
        elif selquit:
            screen.blit(imgbtnquit, sfrquit.topleft)
        elif seleasy:
            screen.blit(imgbtneasy[0], sfreasy.topleft)
        elif selhard:
            screen.blit(imgbtnhard[0], sfrhard.topleft)
        #end if
        
        pygame.display.flip()
    #wend

    # init game
    #--------------------------------
    # frame field
    frames = []
    buckets = []
    for x in range(FX):
        buckets.append([])
        for y in range(FY):
            buckets[x].append(False)
        #next y
    #next x

    # generate complete frame field
    x = 0
    y = 0
    while True:
        size = choice(SIZES)
        b = free_buckets(buckets, x, y, size)
        if b != -1:
            if b != 0:
                size = b
            #end if
            while free_buckets(buckets, x, y, size) != -1:
                 size -= 1
            #wend
            if size == 0:
                x += 1
                if x == FX:
                    x = 0
                    y += 1
                    if y == FY: break
                #end if
                continue
            #end if
        #end if

        fill_buckets(buckets, x, y, size)
        frames.append(Frame(PADX+x*18, PADY+y*24, 3*size))
        x += size
    #wend

    # trim and distribute rogue frames
    shuffle(frames)
    if gamemode == "easy": # 0.8, 0.7
        z = int(len(frames)*0.6) # percentage of frames on screen
        l = int(z*0.85) # percentage of frames at start vs. rogue frames
    else:
        # hard mode
        z = int(len(frames))
        l = int(z*0.6)
    #end if
    oframes = frames[l:z]
    frames = frames[:l]
    dframes = []

    for fr in oframes:
        R = randint(128, 192)
        fr.color = (randint(128, 192), R, R/2)
        fr.other = True
    #next

    selected = []
    done = False
    win = False
    easymode = (gamemode == "easy")

    # main game loop
    cont = True
    while cont and notexit:
        for evt in pygame.event.get():
            if evt.type == QUIT:
                notexit = False
            elif evt.type == KEYDOWN and evt.key == K_ESCAPE:
                cont = False
            elif evt.type == MOUSEBUTTONDOWN:
                # if game is over, just go back to menu
                if done:
                    cont = False
                    break
                #end if
                
                # check for click on each frame
                for fr in frames:
                    if fr.rect.collidepoint(evt.pos):
                        if fr.selected:
                            selected.remove(fr)
                            fr.selected = False
                        else:
                            selected.append(fr)
                            fr.selected = True
                        #end if
                        break
                    #end if
                #next fr

                # pair selected
                if len(selected) == 2:
                    # remove both frames
                    selected[0].closing = True
                    selected[0].value = 1
                    selected[0].selected = False
                    selected[1].closing = True
                    selected[1].selected = False
                    selected[1].value = 1

                    # if both an other, bring back a deleted one
                    if selected[0].other == selected[1].other == True and dframes:
                        sndbad.play()
                        fr = choice(dframes)
                        frames.append(fr)
                        dframes.remove(fr)
                    # If weren't the same size, add a new frame
                    elif selected[0].size != selected[1].size and oframes:
                        sndincorrect.play()
                        fr = choice(oframes)
                        frames.append(fr)
                        oframes.remove(fr)
                    else:
                        sndcorrect.play()
                    #end if
                    selected = []
                #end if
            #end if
        #next

        screen.fill( (128, 0, 0) )

        for x in xrange(len(frames)-1, -1, -1):
            fr = frames[x]
            if draw_frame(screen, fr):
                frames[x].closing = False
                dframes.append(frames[x])
                del frames[x]
            #end if
        #next frame
        
        if easymode:
            for fr in oframes:
                draw_frame(screen, fr, outline=True)
            #next frame
        #end if
        
        #end of game
        if len(frames) <= 1 or done:
            if len(oframes) + len(frames) == 0 or win:
                #win
                screen.blit(imgwin, (WIDTH/2 - imgwin.get_width()/2, HEIGHT/2 - imgwin.get_height()/2))
                if not done:
                    win = True
                    # show all frames
                    frames += dframes
                    sndwin.play()
                #end if
            else:
                #lose
                screen.blit(imglose, (WIDTH/2 - imglose.get_width()/2, HEIGHT/2 - imglose.get_height()/2))
                # copy rogue frames to show user
                if not done:
                    frames += oframes
                    sndlose.play()
                #end if
            #end if
            done = True
        #end if

        pygame.display.flip()
    #wend
#wend

# proper cleanup, for posterity
pygame.mixer.music.stop()
pygame.mixer.stop()
pygame.display.quit()
